*********************
* Determinants and heterogeneity of switching costs in IT outsourcing: estimates from firm-level data *
* Christian Peukert *
* European Journal of Information Systems, 2019 *
* https://doi.org/10.1080/0960085X.2018.1529374 *
*********************

* This code reproduces the results reported in Table 2, 3 and 5

set more off
version 11
capture log close
log using logfile_Theo_Daniel.log, replace
global texpath="results"
use "data.dta", clear
use "data_modified.dta", clear


**Some good to haves
//macro drop _all
//sort join_number year

**We need to define global (or local, but global in the code) macros
**First, vendor (t and t+1) and year fixed effects that then enter into the macro "other"



**The main macros used, as done in the paper
global v vswm  //vswm is v, which is the latent value of switching.
global z llogbarg hhi_cont lhhi dnostd dvrepd //To disentangle "z", "ind" and "other", we had to compare table 2 and 3 on pages 24 and 26.
global ind lspec lvex lvex2 logtravelemp swex swex2 logemp //Changes in long run trajectory of the price index for professional services.
global other yd1-yd11 vmatch_1-vmatch_35 lvmatch_1-lvmatch_33 //"Other" are year and vendor specific fixed effects.
global c deltac //p. 13, "We observe accounting data on expendiutre for professional services"


**We need to install some packages:
ssc install utest
ssc install erepost




****************************************************
************Replication of table 2 and 3************
****************************************************


//We start with the pre-stage. The challenge is that the probit estimates of 
//eq (3) in the paper are only identified up to a scale, i.e., beta/sigma. To
//get around this, we need to first estimate sigma and then use this to estimates beta. 


***First stage regression
//The idea now is to obtain an estimate of alpha to help us infer sigma. We wish to infer sigma to be able to disentangle beta, which is the coefficient for switching costs.
//This strategy is needed because the model is only identified up to a scale (alpha/sigma and beta/sigma) in the main probit regression with the latent value of switching (v) as dependent variable.
//We first have to get an estimate for the cost for IT outsourcing expenditures (px) and a closely correlated variable so that the gamma parameter follows.
//Such a correlated variable is the cost for professional services (cx). The dependent variable used is the change in expenditure for professional services.
//Note that e(sample) looks back on the last model estimated. It excludes observations where "variable"=0 and where observations are missing.

probit $v px $z $ind $other //Running this first and then using e(sample) in the next regression is to assure that the shocks needed in the OLS follow (see technical appendix). I.e., we gain comparable estimates of the parameters, specifically alpha.
reg $c px cx $z $other if e(sample), nocons //This gives us a value of the expenditure on professional services. We also get coefficients for px, which is alpha-gamma, and for cx, which is gamma. We can thus infer alpha.
estimates store c

version 10
probit $v px $z $ind $other if e(sample) //We now run the probit model for the latent value of switching (v) where the coefficient for IT outsourcing expenditures (px) is alpha/sigma. Given that the OLS for c allowed us to identify alpha, we can now infer sigma when combined with the result above from c. This is used to derive table 5.
utest lvex lvex2, fieller
utest swex swex2, fieller
version 11

probit $v px $z $ind $other if e(sample) //Same as above; gives us an estimate of the latent value of switching (v). 
global k=e(k)
estimates store v

//This part simply does the above but puts them in a shared table (column 1 and 2 in table 2).
drop smpl
gen smpl=e(sample)
suest c
estimates store crobust
suest v
estimates store vrobust
suest c v
matrix V=e(V)
matrix b=e(b)
erepost b=b V=V
estimates store sandwich

//Some notes: Interpretation of coefficients is, as the author puts it, "limited to their respective sings", so no direct inference about the magnitude can be inferred. For the probit, we would have to look at marginal effects. 


//--------------------------------------------
//Adjusting coefficients and using robust standard errors (white, combined and clustered).
local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"

	qui {
		global nlcom "(alpha: `alpha') (sigma: `sigma')"
		foreach ind in $z $ind {
			global nlcom "${nlcom} (`ind': `sigma'*[v_${v}]`ind')"
		}
	}
global nlcom "$nlcom (_cons: `sigma'*[v_${v}]_cons)"
nlcom ${nlcom}, post
estimates store nlcom_sandwich


//Combined and clustered standard errors (column 3 in table 2).
qui { 
	suest c v,vce(cluster join_number)
	matrix V_1=e(V)
	suest c v,vce(cluster vidt)
	matrix V_2=e(V)
	suest c v,vce(cluster join_number_vidt)
	matrix V_12=e(V)
	matrix b=e(b)
	matrix V=V_1+V_2-V_12
	erepost b=b V=V
	estimates store cluster
}

local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"

//Adjusted coefficients with clustered standard errors (column 1 in table 3).
nlcom ${nlcom}, post
estimates store nlcom_cluster
	matrix B=e(b)


//--------------------------------------------
//Standardizing the coefficients with their standard deviations and formula for non-linear coefficients
//Peukert runs this again to define the local macros alpha and sigma.

qui { 
	suest c v,vce(cluster join_number)
	matrix V_1=e(V)
	suest c v,vce(cluster vidt)
	matrix V_2=e(V)
	suest c v,vce(cluster join_number_vidt)
	matrix V_12=e(V)
	matrix b=e(b)
	matrix V=V_1+V_2-V_12
	erepost b=b V=V
	estimates store cluster
}

local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"

qui{
foreach var in $z $ind {
	qui su `var' if smpl
	local `var'_sd=r(sd)
	local `var'_dummy=r(min)==0 & r(max)==1
}
}

global nlcom_std "(_cons: (`sigma')*[v_${v}]_cons)"
global nlcom_std "${nlcom_std} (llogbarg: 2*`llogbarg_sd'*`sigma'*[v_${v}]llogbarg)"
global nlcom_std "${nlcom_std} (hhi_cont: 2*`hhi_cont_sd'*`sigma'*[v_${v}]hhi_cont)"
global nlcom_std "${nlcom_std} (dvrepd: `dvrepd_sd'*`sigma'*[v_${v}]dvrepd)"
global nlcom_std "${nlcom_std} (dnostd: 2*`dnostd_sd'*`sigma'*[v_${v}]dnostd)"
global nlcom_std "${nlcom_std} (logemp: 2*`logemp_sd'*`sigma'*[v_${v}]logemp)"
global nlcom_std "${nlcom_std} (lhhi: 2*`lhhi_sd'*`sigma'*[v_${v}]lhhi)"
global nlcom_std "${nlcom_std} (lspec: 2*`lspec_sd'*`sigma'*[v_${v}]lspec)"
global nlcom_std "${nlcom_std} (lvex_s: 2*`lvex_sd'*`sigma'*[v_${v}]lvex+2*`lvex_sd'^2*`sigma'*[v_${v}]lvex2)"
global nlcom_std "${nlcom_std} (logtravelemp: 2*`logtravelemp_sd'*`sigma'*[v_${v}]logtravelemp)"
global nlcom_std "${nlcom_std} (swex_s: 2*`swex_sd'*`sigma'*[v_${v}]swex+2*`swex_sd'^2*`sigma'*[v_${v}]swex2)"
		
//Adjusted and standardized coefficients (column 2 in table 3).
nlcom ${nlcom_std}, post
estimates store nlcom_cluster_std


esttab c v sandwich cluster using "table_2.tex", replace wide noomitted order(px cx $order) mtitle("Sandwich" "Sandwich" "Sandwich" "Cluster") indicate("Year fixed effect=y*" "Vendor fixed effect t=vmatch*" "Vendor fixed effect t-1=lvmatch*") se star(* 0.10 ** 0.05 *** 0.01) scalars("r2_a adj. R2" "r2_p pseudo R2") obslast label compress booktabs equations(1:.:1:1,.:1:3:3) eqlabels(c v)
esttab nlcom_cluster nlcom_cluster_std using "table_3.tex", replace wide order(alpha sigma $order) mtitle("Cluster" "Cluster Std") se star(* 0.10 ** 0.05 *** 0.01) obslast label compress booktabs




****************************************************
***************Replication of table 5***************
****************************************************

//The goal here is to estimate average switching costs of switchers and stayers.
//First we define the relevant variables and macros
global ind lspec lvex lvex2 logtravelemp swex swex2 logemp
global s lhhi dnostd dvrepd $ind //This order is done to capture the coefficients for switching costs
drop s
gen s=0
local i=0


//The first line sets the first for coefficients in the matrix B to equal zero, the following coefficients follow, and the last zero is for the constant.
//This is done to match the dimensions of B, which is 1x15.
foreach var in 0 0 0 0 $s 0{
	local i=`i'+1
	di B[1,`i'] "`var'"
	replace s=s-B[1,`i']*`var' if smpl
}

drop srel
gen srel=s/mean_acct_290d
drop srelt
gen srelt=s/mean_acct_290d/vex
drop srelpx
gen srelpx=s/px


label variable s "Absolute"
label variable srel "Relative"
label variable srelt "Relative per Contract Year"
label variable srelpx "Relative $\p_\tau$"
label define vswm 0 "Non-Switchers" 1 "Switchers"
label values vswm vswm

estpost tabstat s srel srelt, stats(mean sd min max p50 skewness kurtosis iqr) by(vswm) columns(s)
esttab using "table_5.tex", replace ////
cells("mean(fmt(%9.4g) label(Mean)) sd(fmt(%9.4g) label(SD)) min(fmt(%9.4g) label(Min)) max(fmt(%9.4g) label(Max)) p50(fmt(%9.4g) label(Median)) iqr(fmt(%9.4g) label(IQR))") ////
nomtitle nonumber booktabs label noobs






****************************************************
***********************APPENDIX*********************
****************************************************

//----------------------------------------------------
//Trial to get table 5 withouth using the code provided

matrix list B
 
gen coef_llogbarg = B[1,3] * -llogbarg
gen coef_hhi_cont = B[1,4] * -hhi_cont
gen coef_lhhi = B[1,5] * -lhhi
gen coef_dnostd = B[1,6] * -dnostd
gen coef_dvrepd = B[1,7] * -dvrepd
gen coef_lspec = B[1,8] * -lspec
gen coef_lvex = B[1,9] * -lvex
gen coef_lvex2 = B[1,10] * -lvex2
gen coef_logtravelemp = B[1,11] * -logtravelemp
gen coef_swex = B[1,12] * -swex
gen coef_swex2 = B[1,13] * -swex2
gen coef_logemp = B[1,14] * -logemp
gen coef_cons = B[1,15] *-1
 
gen s = coef_lhhi + coef_dnostd + coef_dvrepd +coef_lspec + coef_lvex + ///
				coef_lvex2 + coef_logtravelemp + coef_swex + coef_swex2 + ///
				coef_logemp

//----------------------------------------------------
//Table A.1: descriptive statistics
global stats px deltac cx logbarg_cont llogbarg hhi_cont lhhi dnostd dvrepd lspec lvex logtravelemp swex logemp
estpost tabstat px deltac cx logbarg_cont llogbarg hhi_cont lhhi dnostd dvrepd lspec lvex logtravelemp swex logemp, stats(mean sd min max) by(vswm) columns(s)
esttab using "table_A1.tex", replace ////
cells("Mean(fmt(%9.4g) label(Mean)) SD(fmt(%9.4g) label(SD)) Min(fmt(%9.4g) label(Min)) Max(fmt(%9.4g) label(Max))") ////
nomtitle nonumber booktabs label noobs







****************************************************
*****************USER-MADE EXTENSION****************
****************************************************
 
//----------------------------------------------------
//Plot for 2008 crisis motivation
sort year
by year: egen pre=mean(vswm) if year<2008
sort year
by year: egen post=mean(vswm) if year>=2008
 
 
preserve
 
collapse (mean) pre post vswm, by(year)
tsset year
//twoway line vswm year, title("Average probability of switching") xtitle("Year") ytitle("1==Switch, 0==Stay")
 
twoway (line pre post year, lcolor(blue) lwidth(thick) lpattern(solid)) ///
    , title("Average Probability of Switching") ///
    xtitle("Year") ytitle("") ///
    legend(off) ///
    graphregion(color(white)) bgcolor(white) ///
    xtick(2008, grid) //
	 xline(2008, lcolor(black) lpattern(dash)) ///
    scheme(s1color) ///
	vline(2008, lcolor(black) lpattern(solid)) ///
    note("Note: 1==Switch, 0==Stay.", size(small))
	graph save "crisis_1", replace
restore
 
graph use crisis_1
graph export crisis_1.png, replace




//----------------------------------------------------
//We create our dummies and define our macros.
//Interact the post-2007 period with variables that are time variant (i.e., z)
drop crisisy
gen crisisy = 0
replace crisisy=1 if year==2008
replace crisisy=1 if year==2009
replace crisisy=1 if year==
drop crisishhi
gen crisishhi = crisisy*lhhi //relevant market in general
drop crisishhicont
gen crisishhicont = crisisy*hhi_cont //relevant market at signing
drop crisisbarg
gen crisisbarg = crisisy*logbarg //bargaining power of the firm
drop crisisfeatures
gen crisisfeatures = crisisy*dvrepd //more features
drop crisisvisibility
gen crisisvisibility = crisisy*dvrepd //more visibility

global v vswm  //vswm is v, which is the latent value of switching.
global z llogbarg hhi_cont lhhi dnostd dvrepd //To disentangle "z", "ind" and "other", we had to compare table 2 and 3 on pages 24 and 26.
global ind lspec lvex lvex2 logtravelemp swex swex2 logemp //Changes in long run trajectory of the price index for professional services.
global other yd1-yd11 vmatch_1-vmatch_35 lvmatch_1-lvmatch_33 //"Other" are year and vendor specific fixed effects.
global c deltac //p. 13, "We observe accounting data on expendiutre for professional services"
global crisis crisishhi crisishhicont crisisbarg crisisfeatures crisisvisibility
 

probit $v px $z $ind $other $crisis
reg $c px cx $z $other $crisis if e(sample), nocons
estimates store c

version 10
probit $v px $z $ind $other $crisis if e(sample)
utest lvex lvex2, fieller
utest swex swex2, fieller
version 11

probit $v px $z $ind $other $crisis if e(sample)
global k=e(k)
estimates store v

drop smpl
gen smpl=e(sample)
suest c
estimates store crobust
suest v
estimates store vrobust
suest c v
matrix V=e(V)
matrix b=e(b)
erepost b=b V=V
estimates store sandwich


//--------------------------------------------
//Adjusting coefficients and using robust standard errors (white, combined and clustered).
local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"


//We modify on lines 312-314 to include these new interaction terms.
	qui {
		global nlcom "(alpha: `alpha') (sigma: `sigma')"
		foreach ind in $z $ind {
			global nlcom "${nlcom} (`ind': `sigma'*[v_${v}]`ind')"
		}
		foreach crisis in $crisis {
			global nlcom "${nlcom} (`crisis': `sigma'*[v_${v}]`crisis')"
		}
	}
	
global nlcom "$nlcom (_cons: `sigma'*[v_${v}]_cons)"
nlcom ${nlcom}, post
estimates store nlcom_sandwich


//Combined and clustered standard errors (column 3 in table 2).
qui { 
	suest c v,vce(cluster join_number)
	matrix V_1=e(V)
	suest c v,vce(cluster vidt)
	matrix V_2=e(V)
	suest c v,vce(cluster join_number_vidt)
	matrix V_12=e(V)
	matrix b=e(b)
	matrix V=V_1+V_2-V_12
	erepost b=b V=V
	estimates store cluster
}

local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"

//Adjusted coefficients with clustered standard errors (column 1 in table 3).
nlcom ${nlcom}, post
estimates store nlcom_cluster
	matrix B=e(b)


//--------------------------------------------
//Standardizing the coefficients with their standard deviations and formula for non-linear coefficients.
//Peukert runs this again to define the local macros alpha and sigma.

qui { 
	suest c v,vce(cluster join_number)
	matrix V_1=e(V)
	suest c v,vce(cluster vidt)
	matrix V_2=e(V)
	suest c v,vce(cluster join_number_vidt)
	matrix V_12=e(V)
	matrix b=e(b)
	matrix V=V_1+V_2-V_12
	erepost b=b V=V
	estimates store cluster
}

local alpha="[c_mean]px+[c_mean]cx"
local sigma="(`alpha')/[v_${v}]px"


//We make sure that the $crisis macro is included.
qui{
foreach var in $z $ind $crisis {
	qui su `var' if smpl
	local `var'_sd=r(sd)
	local `var'_dummy=r(min)==0 & r(max)==1
}
}

//The interaction terms in $crisis are adjusted and standardized
global nlcom_std "(_cons: (`sigma')*[v_${v}]_cons)"
global nlcom_std "${nlcom_std} (llogbarg: 2*`llogbarg_sd'*`sigma'*[v_${v}]llogbarg)"
global nlcom_std "${nlcom_std} (hhi_cont: 2*`hhi_cont_sd'*`sigma'*[v_${v}]hhi_cont)"
global nlcom_std "${nlcom_std} (dvrepd: `dvrepd_sd'*`sigma'*[v_${v}]dvrepd)"
global nlcom_std "${nlcom_std} (dnostd: 2*`dnostd_sd'*`sigma'*[v_${v}]dnostd)"
global nlcom_std "${nlcom_std} (logemp: 2*`logemp_sd'*`sigma'*[v_${v}]logemp)"
global nlcom_std "${nlcom_std} (lhhi: 2*`lhhi_sd'*`sigma'*[v_${v}]lhhi)"
global nlcom_std "${nlcom_std} (lspec: 2*`lspec_sd'*`sigma'*[v_${v}]lspec)"
global nlcom_std "${nlcom_std} (crisishhi: 2*`crisishhi_sd'*`sigma'*[v_${v}]crisishhi)" //for lhhi post 2008
global nlcom_std "${nlcom_std} (crisishhicont: 2*`crisishhicont_sd'*`sigma'*[v_${v}]crisishhicont)" //for hhi_cont post 2008
global nlcom_std "${nlcom_std} (crisisbarg: 2*`crisisbarg_sd'*`sigma'*[v_${v}]crisisbarg)" //for logbarg post 2008
global nlcom_std "${nlcom_std} (crisisfeatures: 2*`crisisfeatures_sd'*`sigma'*[v_${v}]crisisfeatures)" //for dvrepd post 2008
global nlcom_std "${nlcom_std} (crisisvisibility: 2*`crisisvisibility_sd'*`sigma'*[v_${v}]crisisvisibility)" //for dnostd post 2008
global nlcom_std "${nlcom_std} (lvex_s: 2*`lvex_sd'*`sigma'*[v_${v}]lvex+2*`lvex_sd'^2*`sigma'*[v_${v}]lvex2)"
global nlcom_std "${nlcom_std} (logtravelemp: 2*`logtravelemp_sd'*`sigma'*[v_${v}]logtravelemp)"
global nlcom_std "${nlcom_std} (swex_s: 2*`swex_sd'*`sigma'*[v_${v}]swex+2*`swex_sd'^2*`sigma'*[v_${v}]swex2)"
		
//Adjusted and standardized coefficients (column 2 in table 3)
nlcom ${nlcom_std}, post
estimates store nlcom_cluster_std


esttab c v sandwich cluster using "table_2_mod.tex", replace wide noomitted order(px cx $order) mtitle("Sandwich" "Sandwich" "Sandwich" "Cluster") indicate("Year fixed effect=y*" "Vendor fixed effect t=vmatch*" "Vendor fixed effect t-1=lvmatch*") se star(* 0.10 ** 0.05 *** 0.01) scalars("r2_a adj. R2" "r2_p pseudo R2") obslast label compress booktabs equations(1:.:1:1,.:1:3:3) eqlabels(c v)
esttab nlcom_cluster nlcom_cluster_std using "table_3_mod.tex", replace wide order(alpha sigma $order) mtitle("Cluster" "Cluster Std") se star(* 0.10 ** 0.05 *** 0.01) obslast label compress booktabs


log close
